#include <bits/stdc++.h>

#include <utility>

using namespace std;

#define all(x) (x).begin(), (x).end()
using ll = long long;

struct plan_segment {
    int l, r;
    int h;

    plan_segment(int i, int h):
            l(i), r(i), h(h) {}

    [[nodiscard]] ll value() const {
        return static_cast<ll>(r - l + 1) * h;
    }
};

struct plan_block {
    vector<plan_segment> pref_max;
    vector<plan_segment> suff_max;
    int l;
    int r;
    ll answer_pref;
    ll answer_suff;
    ll sum;

    plan_block(int i, int h) {
        pref_max.emplace_back(i, h);
        suff_max.emplace_back(i, h);
        l = i;
        r = i;
        answer_pref = h;
        answer_suff = h;
        sum = h;
    }

    void add_seg_to_pref_left(plan_segment seg) {
        while (!pref_max.empty() && pref_max.front().h <= seg.h) {
            answer_pref -= pref_max.front().value();

            seg.r = pref_max.front().r;

            pref_max.erase(pref_max.begin());
        }
        answer_pref += seg.value();
        pref_max.insert(pref_max.begin(), seg);
    }

    void add_seg_to_pref_right(plan_segment seg) {
        if (pref_max.back().h >= seg.h) {
            answer_pref -= pref_max.back().value();

            pref_max.back().r = seg.r;

            answer_pref += pref_max.back().value();
        } else {
            answer_pref += seg.value();
            pref_max.push_back(seg);
        }
    }

    void add_seg_to_suff_left(plan_segment seg) {
        if (suff_max.front().h >= seg.h) {
            answer_suff -= suff_max.front().value();

            suff_max.front().l = seg.l;

            answer_suff += suff_max.front().value();
        } else {
            answer_suff += seg.value();
            suff_max.insert(suff_max.begin(), seg);
        }
    }

    void add_seg_to_suff_right(plan_segment seg) {
        while (!suff_max.empty() && suff_max.back().h <= seg.h) {
            answer_suff -= suff_max.back().value();

            seg.l = suff_max.back().l;

            suff_max.pop_back();
        }
        answer_suff += seg.value();
        suff_max.push_back(seg);
    }

    void move_from(plan_block &other) {
        if (other.l == r + 1) {
            while (!other.pref_max.empty()) {
                add_seg_to_pref_right(other.pref_max.front());
                other.pref_max.erase(other.pref_max.begin());
            }
            while (!other.suff_max.empty()) {
                add_seg_to_suff_right(other.suff_max.front());
                other.suff_max.erase(other.suff_max.begin());
            }
            r = other.r;
            sum += other.sum;
        } else if (other.r + 1 == l) {
            while (!other.pref_max.empty()) {
                add_seg_to_pref_left(other.pref_max.back());
                other.pref_max.pop_back();
            }
            while (!other.suff_max.empty()) {
                add_seg_to_suff_left(other.suff_max.back());
                other.suff_max.pop_back();
            }
            l = other.l;
            sum += other.sum;
        } else {
            cerr << "Blocks are not consequent!" << endl;
            exit(1);
        }
    }

    ll get_puddleness() {
        return answer_suff + answer_pref - static_cast<ll>(r - l + 1) * pref_max.back().h - sum;
    }

    int len() {
        return r - l + 1;
    }
};

struct dsu {
    vector<int> par;
    vector<optional<plan_block>> tree_blocks; // invariant - Some(plan_block) only for leaders

    explicit dsu(const vector<int> &h) {
        int n = h.size();
        par.resize(n, -1);
        tree_blocks.resize(n, nullopt);
        for (int i = 0; i < n; i++) {
            tree_blocks[i] = plan_block(i, h[i]);
        }
    }

    int lead(int v) {
        if (par[v] == -1) {
            return v;
        }
        par[v] = lead(par[v]);
        return par[v];
    }

    ll merge2(int i, int j) {
        i = lead(i);
        j = lead(j);
        if (tree_blocks[i].value().len() < tree_blocks[j].value().len()) {
            swap(i, j);
        }
        tree_blocks[i].value().move_from(tree_blocks[j].value());
        tree_blocks[j] = nullopt;
        par[j] = i;
        return tree_blocks[i].value().get_puddleness();
    }

    ll merge(int l, int r) {
        ll res;
        if (lead(l) == lead(r)) {
            return tree_blocks[lead(l)].value().get_puddleness();
        }
        while (l < r) {
            int lead_r = tree_blocks[lead(l)].value().r;
            if (lead_r + 1 > r) {
                break;
            }
            res = merge2(l, lead_r + 1);
            l = lead_r + 1;
        }
        return res;
    }

    ll get_for_block(int i) {
        i = lead(i);
        return tree_blocks[i].value().get_puddleness();
    }
};

namespace cartesian {
    mt19937 rnd(chrono::steady_clock::now().time_since_epoch().count());

    int rand(int l, int r) {
        uniform_int_distribution x(l, r);
        return x(rnd);
    }


    struct Node {
        int y;
        int sz;
        pair<int, int> range;
        Node *left, *right;

        explicit Node(int x):
                y(rand(INT_MIN, INT_MAX)),
                sz(1), range(make_pair(x, x)),
                left(nullptr), right(nullptr) {}

        explicit Node(pair<int, int> range):
                y(rand(INT_MIN, INT_MAX)),
                sz(1), range(std::move(range)),
                left(nullptr), right(nullptr) {}
    };

    int size(Node *v) {
        if (!v) {
            return 0;
        }
        return v->sz;
    }

    void recalc(Node *v) {
        if (!v) return;
        v->sz = size(v->left) + size(v->right) + 1;
    }

    Node* merge(Node *l, Node *r) {
        if (!l) {
            return r;
        }
        if (!r) {
            return l;
        }
        if (l->y < r->y) {
            l->right = merge(l->right, r);
            recalc(l);
            return l;
        } else {
            r->left = merge(l, r->left);
            recalc(r);
            return r;
        }
    }

    // k elements in left
    pair<Node*, Node*> split(Node *t, int k) {
        if (!t) {
            return {nullptr, nullptr};
        }
        if (size(t->left) + 1 <= k) {
            auto r = split(t->right, k - 1 - size(t->left));
            t->right = r.first;
            recalc(t);
            return {t, r.second};
        } else {
            auto l = split(t->left, k);
            t->left = l.second;
            recalc(t);
            return {l.first, t};
        }
    }

    Node* get_first(Node *t) {
        if (!t->left) {
            return t;
        }
        return get_first(t->left);
    }

    Node* get_last(Node *t) {
        if (!t->right) {
            return t;
        }
        return get_last(t->right);
    }

    void delete_tree(Node *t) {
        if (!t) return;
        delete_tree(t->left);
        delete_tree(t->right);
        delete t;
    }
}

struct plan_tracker {
    cartesian::Node *root = nullptr;

    explicit plan_tracker(int n) {
        for (int i = 0; i < n; i++) {
            root = merge(root, new cartesian::Node(i));
        }
    }

    pair<int, int> unite_all(int l, int r) {
        auto r_split = cartesian::split(root, r);
        auto l_split = cartesian::split(r_split.first, l - 1);

        int real_l = cartesian::get_first(l_split.second)->range.first;
        int real_r = cartesian::get_last(l_split.second)->range.second;

        cartesian::delete_tree(l_split.second);
        auto new_node = new cartesian::Node(make_pair(real_l, real_r));

        root = cartesian::merge(l_split.first, cartesian::merge(new_node, r_split.second));
        return make_pair(real_l, real_r);
    }
};

void solve() {
    int n;
    cin >> n;
    vector<int> h(n);
    for (auto &x : h) {
        cin >> x;
    }
    dsu blocks(h);
    plan_tracker tracker(n);

    int m = n - 1;
    while (m--) {
        int l, r;
        cin >> l;
        r = l + 1;
        auto real_indexes = tracker.unite_all(l, r);
        cout << blocks.merge(real_indexes.first, real_indexes.second) << "\n";
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    solve();
    return 0;
}
