#include <bits/stdc++.h>
#include <algorithm>
#include <cctype>
#include "cli_args.hpp"
#include "testlib.h"

using namespace std;
using pi = pair<int, int>;

CLIArguments args;

template <typename T>
void Shuffle(vector<T>& v) {
    for (int i = 1; i < (int)v.size(); ++i) {
        swap(v[rnd.next(0, i)], v[i]);
    }
}

void AppendRandomPath(vector<pi>& cand, int n, int m) {
    vector<int> moves(n + m - 2);
    fill(moves.begin(), moves.begin() + n - 1, 1);
    Shuffle(moves);

    int cx = 1, cy = 1;
    cand.emplace_back(1, 1);
    for (int j : moves) {
        cand.emplace_back(cx += j, cy += 1 - j);
    }
}

void PrintTestCase(const int n, const int m, const vector<pi>& cand) {
    println(n, m, cand.size());
    for (auto [x, y] : cand) {
        println(x, y);
    }
}

void GenTwoPath() {
    const int n = args.GetInt("n");
    const int m = args.GetInt("m");
    int k = args.GetInt("k");

    vector<pi> cand;
    for (int path_id = 0; path_id < 2; ++path_id) {
        AppendRandomPath(cand, n, m);
    }

    sort(cand.begin(), cand.end());
    cand.erase(unique(cand.begin(), cand.end()), cand.end());
    Shuffle(cand);

    k = min<int>(k, cand.size());
    cand.resize(k);

    PrintTestCase(n, m, cand);
}

void GenSinglePath() {
    const int n = args.GetInt("n");
    const int m = args.GetInt("m");
    int k = args.GetInt("k");

    vector<pi> cand;
    AppendRandomPath(cand, n, m);

    sort(cand.begin(), cand.end());
    cand.erase(unique(cand.begin(), cand.end()), cand.end());
    Shuffle(cand);

    k = min<int>(k, cand.size());
    cand.resize(k);

    PrintTestCase(n, m, cand);
}

void GenCorner() {
    const int n = args.GetInt("n");
    const int m = args.GetInt("m");

    string s = args.GetStr("s");
    for (char& c : s)
        c = toupper(c);

    int x = 1, y = 1;
    ensuref(s.size() == 2, "incorrect corner");
    ensuref(s[0] == 'L' || s[0] == 'R', "format s=<L|R><U|D>");
    ensuref(s[1] == 'U' || s[1] == 'D', "format s=<L|R><U|D>");

    if (s[0] == 'R')
        y = m;
    if (s[1] == 'D')
        x = n;

    vector<pi> cand{{pi{x, y}}};
    PrintTestCase(n, m, cand);
}

bool IsInversion(const pi& a, const pi& b) {
    const auto [ax, ay] = a;
    const auto [bx, by] = b;
    return (ax < bx && ay > by) || (ax > bx && ay < by);
}

bool IsImpossible(const vector<pi>& v) {
    const int k = v.size();
    for (int i = 0; i < k; ++i) {
        for (int j = 0; j < i; ++j) {
            if (!IsInversion(v[i], v[j]))
                continue;
            for (int t = 0; t < j; ++t) {
                if (IsInversion(v[i], v[t]) && IsInversion(v[j], v[t]))
                    return true;
            }
        }
    }
    return false;
}

void GenRandomPoints() {
    const int n = args.GetInt("n");
    const int m = args.GetInt("m");
    const int k = args.GetInt("k");
    const string loc = args.GetStr("loc");

    vector<pi> cand(k);
    while (true) {
        for (int i = 0; i < k; ++i) {
            auto& [x, y] = cand[i];

            while (true) {
                x = rnd.next(1, n);
                y = rnd.next(1, m);

                bool same = false;
                for (int j = 0; j < i; ++j) {
                    if (cand[i] == cand[j]) {
                        same = true;
                        break;
                    }
                }
                if (!same) break;
            }
        }

        bool found_same = false;
        for (int i = 0; i < k; ++i) {
            for (int j = 0; j < i; ++j) {
                if (cand[i] == cand[j]) {
                    found_same = true;
                    break;
                }
            }
            if (found_same)
                break;
        }
        if (found_same)
            continue;

        if (loc == "impossible") {
            ensuref(k >= 3,
                    "for loc=impossible k should be equal or greater than 3");
            if (IsImpossible(cand))
                break;
        } else if (loc == "possible") {
            ensuref(k <= 16, "for loc=possible k should be not to big");
            if (!IsImpossible(cand))
                break;
        } else {
            ensuref(false, "unknown loc value");
        }
    }

    PrintTestCase(n, m, cand);
}

void GenPairsPath() {
    const int n = args.GetInt("n");
    const int m = args.GetInt("m");
    const int k = args.GetInt("k");
    ensuref(k % 2 == 0, "k should be even");

    vector<pi> corners;
    AppendRandomPath(corners, n - 1, m - 1);
    Shuffle(corners);
    corners.resize(min<int>(corners.size(), k / 2));

    vector<pi> cand;
    for (auto [x, y] : corners) {
        cand.emplace_back(x, y + 1);
        cand.emplace_back(x + 1, y);
    }
    Shuffle(cand);

    PrintTestCase(n, m, cand);
}

void GenShortDiagonalCut() {
    const int n = args.GetInt("n");
    const int k = args.GetInt("k");
    const string cut = args.GetStr("cut");

    vector<int> x;

    if (cut == "random") {
        x.resize(n);
        iota(x.begin(), x.end(), 1);
        Shuffle(x);
        x.resize(min<int>(x.size(), k));
    } else if (cut == "prefix") {
        for (int i = 1; i <= k; ++i)
            x.push_back(i);
    } else if (cut == "suffix") {
        for (int i = 1; i <= k; ++i)
            x.push_back(n - i + 1);
    } else {
        ensuref(false, "unknown cut");
    }

    vector<pi> cand;
    for (int t : x) {
        cand.emplace_back(t, t);
    }

    PrintTestCase(n, n, cand);
}

int main(int argc, char* argv[]) {
    ios::sync_with_stdio(0);
    cin.tie(0);
    registerGen(argc, argv, 1);
    args.Parse(argc, argv);

    auto type = args.GetStr("type");
    if (type == "random") {
        GenTwoPath();
    } else if (type == "single") {
        GenSinglePath();
    } else if (type == "corner") {
        GenCorner();
    } else if (type == "random-points") {
        GenRandomPoints();
    } else if (type == "pairs-path") {
        GenPairsPath();
    } else if (type == "diagonal") {
        GenShortDiagonalCut();
    } else {
        quitf(_fail, "unknown type generator");
    }
}
