#include <set>
#include <utility>
#include "testlib.h"

using namespace std;

constexpr int N = 1'000'000;
constexpr int M = 1'000'000;
constexpr int K = 2'000;

constexpr int GROUP_COUNT = 11;
std::array<std::map<std::string, int>, GROUP_COUNT> additional_constraints{
    {{},
     {{"min_n", 3}, {"n", 3}, {"m", 100}, {"k", 100}},
     {{"n", 5}, {"m", 5}, {"k", 5}},
     {{"n", 8}, {"m", 8}, {"k", 8}},
     {{"n", 30}, {"m", 30}, {"k", 30}},
     {{"n", 100}, {"m", 100}, {"k", 100}},
     {{"k", 0}},
     {{"k", 1}, {"min_k", 1}},
     {{"k", 16}},
     {{"k", 100}},
     {}}};

int main(int argc, char* argv[]) {
    ios::sync_with_stdio(0);
    cin.tie(0);
    registerValidation(argc, argv);

    const int group = std::stoi(validator.group());
    ensure(group < GROUP_COUNT);
    const auto& constraints = additional_constraints[group];

    auto get = [&constraints](const std::string& idx, const int default_value) {
        auto it = constraints.find(idx);
        return it == constraints.end() ? default_value : it->second;
    };

    const int min_n = get("min_n", 3);
    const int max_n = get("n", N);
    const int min_m = get("min_m", 3);
    const int max_m = get("m", M);
    const int min_k = get("min_k", 0);
    const int max_k = get("k", K);

    const int n = inf.readInt(min_n, max_n, "n");
    inf.readSpace();
    const int m = inf.readInt(min_m, max_m, "m");
    inf.readSpace();
    const int k = inf.readInt(min_k, max_k, "k");
    inf.readEoln();

    std::set<std::pair<int, int>> pts;
    for (int i = 1; i <= k; ++i) {
        const int x = inf.readInt(1, n, "x_" + to_string(i));
        inf.readSpace();
        const int y = inf.readInt(1, m, "y_" + to_string(i));
        inf.readEoln();

        std::pair<int, int> pt{x, y};
        ensuref(!pts.count(pt), "all points should be unique");
        pts.insert(pt);
    }

    inf.readEof();
}
