#include <bits/stdc++.h>

using namespace std;

/*
 ! WARNING: MOD must be prime.
 ! WARNING: MOD must be less than 2^30.
 * Use .get() to get the stored value.
 */

template <uint32_t mod>
class montgomery {
    static_assert(mod < uint32_t(1) << 30, "mod < 2^30");
    using mint = montgomery<mod>;

   private:
    uint32_t value;

    static constexpr uint32_t inv_neg_mod = []() {
        uint32_t x = mod;
        for (int i = 0; i < 4; ++i) {
            x *= uint32_t(2) - mod * x;
        }
        return -x;
    }();
    static_assert(mod * inv_neg_mod == -1);

    static constexpr uint32_t neg_mod = (-uint64_t(mod)) % mod;

    static uint32_t reduce(const uint64_t& value) {
        return (value + uint64_t(uint32_t(value) * inv_neg_mod) * mod) >> 32;
    }

    inline static const mint ONE = mint(1);

   public:
    montgomery() : value(0) {}
    montgomery(const mint& x) : value(x.value) {}

    template <typename T,
              typename U = std::enable_if_t<std::is_integral<T>::value>>
    montgomery(const T& x)
        : value(
              !x ? 0
                 : reduce(int64_t(x % int32_t(mod) + int32_t(mod)) * neg_mod)) {
    }

    static constexpr uint32_t get_mod() { return mod; }

    uint32_t get() const {
        auto real_value = reduce(value);
        return real_value < mod ? real_value : real_value - mod;
    }

    template <typename T>
    mint power(T degree) const {
        degree =
            (degree % int32_t(mod - 1) + int32_t(mod - 1)) % int32_t(mod - 1);
        mint prod = 1, a = *this;
        for (; degree > 0; degree >>= 1, a *= a)
            if (degree & 1)
                prod *= a;

        return prod;
    }

    mint inv() const { return power(-1); }

    mint& operator=(const mint& x) {
        value = x.value;
        return *this;
    }

    mint& operator+=(const mint& x) {
        if (int32_t(value += x.value - (mod << 1)) < 0) {
            value += mod << 1;
        }
        return *this;
    }

    mint& operator-=(const mint& x) {
        if (int32_t(value -= x.value) < 0) {
            value += mod << 1;
        }
        return *this;
    }

    mint& operator*=(const mint& x) {
        value = reduce(uint64_t(value) * x.value);
        return *this;
    }

    mint& operator/=(const mint& x) { return *this *= x.inv(); }

    friend mint operator+(const mint& x, const mint& y) { return mint(x) += y; }

    friend mint operator-(const mint& x, const mint& y) { return mint(x) -= y; }

    friend mint operator*(const mint& x, const mint& y) { return mint(x) *= y; }

    friend mint operator/(const mint& x, const mint& y) { return mint(x) /= y; }

    mint& operator++() { return *this += ONE; }

    mint& operator--() { return *this -= ONE; }

    mint operator++(int) {
        mint prev = *this;
        *this += ONE;
        return prev;
    }

    mint operator--(int) {
        mint prev = *this;
        *this -= ONE;
        return prev;
    }

    mint operator-() const { return mint(0) - *this; }

    bool operator==(const mint& x) const { return get() == x.get(); }

    bool operator!=(const mint& x) const { return get() != x.get(); }

    bool operator<(const mint& x) const { return get() < x.get(); }

    template <typename T>
    explicit operator T() {
        return get();
    }

    friend std::istream& operator>>(std::istream& in, mint& x) {
        std::string s;
        in >> s;
        x = 0;
        bool neg = s[0] == '-';
        for (const auto c : s)
            if (c != '-')
                x = x * 10 + (c - '0');

        if (neg)
            x *= -1;

        return in;
    }

    friend std::ostream& operator<<(std::ostream& out, const mint& x) {
        return out << x.get();
    }
};

constexpr uint32_t MOD = 1'000'000'007;
// constexpr uint32_t MOD = 998'244'353;
using mint = montgomery<MOD>;
using ll = long long;

/*
 ! WARNING: MOD must be prime.
 * Define modular int class above it.
 * No need to run any init function, it dynamically resizes the data.
 */
namespace combinatorics {
std::vector<mint> fact_, ifact_, inv_;

void resize_data(int size) {
    if (fact_.empty()) {
        fact_ = {mint(1), mint(1)};
        ifact_ = {mint(1), mint(1)};
        inv_ = {mint(0), mint(1)};
    }
    for (int pos = fact_.size(); pos <= size; pos++) {
        fact_.push_back(fact_.back() * mint(pos));
        inv_.push_back(-inv_[MOD % pos] * mint(MOD / pos));
        ifact_.push_back(ifact_.back() * inv_[pos]);
    }
}

struct combinatorics_info {
    std::vector<mint>& data;

    combinatorics_info(std::vector<mint>& data) : data(data) {}

    mint operator[](int pos) {
        if (pos >= static_cast<int>(data.size())) {
            resize_data(pos);
        }
        return data[pos];
    }
} fact(fact_), ifact(ifact_), inv(inv_);

// From n choose k.
// O(max(n)) in total.
mint choose(int n, int k) {
    if (n < k || k < 0 || n < 0) {
        return mint(0);
    }
    return fact[n] * ifact[k] * ifact[n - k];
}
}  // namespace combinatorics

using namespace combinatorics;
using pi = pair<int, int>;

// adds starts and finishes, removes duplicates and sort array of points
void make_great_again(int n, int m, vector<pi>& pts) {
    pts.emplace_back(1, 2);
    pts.emplace_back(2, 1);
    pts.emplace_back(n - 1, m);
    pts.emplace_back(n, m - 1);

    vector<pi> tmp;
    tmp.swap(pts);
    for (auto [x, y] : tmp) {
        if (x == 1 && y == 1)
            continue;
        if (x == n && y == m)
            continue;
        pts.emplace_back(x, y);
    }

    sort(pts.begin(), pts.end());
    pts.erase(unique(pts.begin(), pts.end()), pts.end());
}

mint path_count(pi S, pi T) {
    int x = T.first - S.first;
    int y = T.second - S.second;
    return choose(x + y, x);
}

mint solve(int n, int m, vector<pi> pts) {
    make_great_again(n, m, pts);

    const int k = pts.size();
    const int s1 = find(pts.begin(), pts.end(), pi{1, 2}) - pts.begin();
    const int t1 = find(pts.begin(), pts.end(), pi{n - 1, m}) - pts.begin();
    const int s2 = find(pts.begin(), pts.end(), pi{2, 1}) - pts.begin();
    const int t2 = find(pts.begin(), pts.end(), pi{n, m - 1}) - pts.begin();
    if (min(s1, s2) != 0) return 0; // should be impossible but okay
    if (max(t1, t2) != k-1) return 0; // should be impossible but okay

    vector<vector<mint>> dp(k, vector<mint>(k));
    
    mint start = 2;
    for (int i = min(s1, s2); i + 1 < max(s1, s2); ++i) {
        start *= path_count(pts[i], pts[i+1]);
    }
    const int uw = max(s1, s2) - 1;
    dp[max(uw, s1)][max(uw, s2)] = start;

    for (int i = 0; i < k - 1; ++i) {
        for (int j = 0; j < k - 1; ++j) {
            const int t = max(i, j) + 1;
            dp[i][t] += dp[i][j] * path_count(pts[j], pts[t]);
            dp[t][j] += dp[i][j] * path_count(pts[i], pts[t]);
        }
    }

    return dp[t1][t2] - dp[t2][t1];
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int n, m, k;
    cin >> n >> m >> k;
    vector<pi> pts(k);
    for (auto& [x, y] : pts) {
        cin >> x >> y;
    }

    auto ans = solve(n, m, pts);
    cout << ans << '\n';
}